#define SYS_ __NR_
#define SYS_accept __NR_accept
#define SYS_accept4 __NR_accept4
#define SYS_access __NR_access
#define SYS_acct __NR_acct
#define SYS_acl_get __NR_acl_get
#define SYS_acl_set __NR_acl_set
#define SYS_add_key __NR_add_key
#define SYS_adjtimex __NR_adjtimex
#define SYS_afs_syscall __NR_afs_syscall
#define SYS_alarm __NR_alarm
#define SYS_alloc_hugepages __NR_alloc_hugepages
#define SYS_aplib __NR_aplib
#define SYS_arch_prctl __NR_arch_prctl
#define SYS_arch_specific_syscall __NR_arch_specific_syscall
#define SYS_arm_fadvise64_64 __NR_arm_fadvise64_64
#define SYS_arm_sync_file_range __NR_arm_sync_file_range
#define SYS_attrctl __NR_attrctl
#define SYS_bdflush __NR_bdflush
#define SYS_bind __NR_bind
#define SYS_bpf __NR_bpf
#define SYS_break __NR_break
#define SYS_brk __NR_brk
#define SYS_cachectl __NR_cachectl
#define SYS_cacheflush __NR_cacheflush
#define SYS_capget __NR_capget
#define SYS_capset __NR_capset
#define SYS_chdir __NR_chdir
#define SYS_chmod __NR_chmod
#define SYS_chown __NR_chown
#define SYS_chown32 __NR_chown32
#define SYS_chroot __NR_chroot
#define SYS_clock_adjtime __NR_clock_adjtime
#define SYS_clock_adjtime64 __NR_clock_adjtime64
#define SYS_clock_getres __NR_clock_getres
#define SYS_clock_getres_time64 __NR_clock_getres_time64
#define SYS_clock_gettime __NR_clock_gettime
#define SYS_clock_gettime64 __NR_clock_gettime64
#define SYS_clock_nanosleep __NR_clock_nanosleep
#define SYS_clock_nanosleep_time64 __NR_clock_nanosleep_time64
#define SYS_clock_settime __NR_clock_settime
#define SYS_clock_settime64 __NR_clock_settime64
#define SYS_clone __NR_clone
#define SYS_clone2 __NR_clone2
#define SYS_clone3 __NR_clone3
#define SYS_close __NR_close
#define SYS_close_range __NR_close_range
#define SYS_connect __NR_connect
#define SYS_copy_file_range __NR_copy_file_range
#define SYS_creat __NR_creat
#define SYS_create_module __NR_create_module
#define SYS_delete_module __NR_delete_module
#define SYS_dipc __NR_dipc
#define SYS_dup __NR_dup
#define SYS_dup2 __NR_dup2
#define SYS_dup3 __NR_dup3
#define SYS_epoll_create __NR_epoll_create
#define SYS_epoll_create1 __NR_epoll_create1
#define SYS_epoll_ctl __NR_epoll_ctl
#define SYS_epoll_ctl_old __NR_epoll_ctl_old
#define SYS_epoll_pwait __NR_epoll_pwait
#define SYS_epoll_pwait2 __NR_epoll_pwait2
#define SYS_epoll_wait __NR_epoll_wait
#define SYS_epoll_wait_old __NR_epoll_wait_old
#define SYS_eventfd __NR_eventfd
#define SYS_eventfd2 __NR_eventfd2
#define SYS_execv __NR_execv
#define SYS_execve __NR_execve
#define SYS_execveat __NR_execveat
#define SYS_exec_with_loader __NR_exec_with_loader
#define SYS_exit __NR_exit
#define SYS_exit_group __NR_exit_group
#define SYS_faccessat __NR_faccessat
#define SYS_faccessat2 __NR_faccessat2
#define SYS_fadvise64 __NR_fadvise64
#define SYS_fadvise64_64 __NR_fadvise64_64
#define SYS_fallocate __NR_fallocate
#define SYS_fanotify_init __NR_fanotify_init
#define SYS_fanotify_mark __NR_fanotify_mark
#define SYS_fchdir __NR_fchdir
#define SYS_fchmod __NR_fchmod
#define SYS_fchmodat __NR_fchmodat
#define SYS_fchown __NR_fchown
#define SYS_fchown32 __NR_fchown32
#define SYS_fchownat __NR_fchownat
#define SYS_fcntl __NR_fcntl
#define SYS_fcntl64 __NR_fcntl64
#define SYS_fdatasync __NR_fdatasync
#define SYS_fgetxattr __NR_fgetxattr
#define SYS_finit_module __NR_finit_module
#define SYS_flistxattr __NR_flistxattr
#define SYS_flock __NR_flock
#define SYS_fork __NR_fork
#define SYS_free_hugepages __NR_free_hugepages
#define SYS_fremovexattr __NR_fremovexattr
#define SYS_fsconfig __NR_fsconfig
#define SYS_fsetxattr __NR_fsetxattr
#define SYS_fsmount __NR_fsmount
#define SYS_fsopen __NR_fsopen
#define SYS_fspick __NR_fspick
#define SYS_fstat __NR_fstat
#define SYS_fstat64 __NR_fstat64
#define SYS_fstatat __NR_fstatat
#define SYS_fstatat64 __NR_fstatat64
#define SYS_fstatfs __NR_fstatfs
#define SYS_fstatfs64 __NR_fstatfs64
#define SYS_fsync __NR_fsync
#define SYS_ftime __NR_ftime
#define SYS_ftruncate __NR_ftruncate
#define SYS_ftruncate64 __NR_ftruncate64
#define SYS_futex __NR_futex
#define SYS_futex_time64 __NR_futex_time64
#define SYS_futimesat __NR_futimesat
#define SYS_getcpu __NR_getcpu
#define SYS_getcwd __NR_getcwd
#define SYS_getdents __NR_getdents
#define SYS_getdents64 __NR_getdents64
#define SYS_getdirentires __NR_getdirentires
#define SYS_getdirentries __NR_getdirentries
#define SYS_getdomainname __NR_getdomainname
#define SYS_getdtablesize __NR_getdtablesize
#define SYS_getegid __NR_getegid
#define SYS_getegid32 __NR_getegid32
#define SYS_geteuid __NR_geteuid
#define SYS_geteuid32 __NR_geteuid32
#define SYS_getgid __NR_getgid
#define SYS_getgid32 __NR_getgid32
#define SYS_getgroups __NR_getgroups
#define SYS_getgroups32 __NR_getgroups32
#define SYS_gethostname __NR_gethostname
#define SYS_getitimer __NR_getitimer
#define SYS_get_kernel_syms __NR_get_kernel_syms
#define SYS_get_mempolicy __NR_get_mempolicy
#define SYS_getmsg __NR_getmsg
#define SYS_getpagesize __NR_getpagesize
#define SYS_getpeername __NR_getpeername
#define SYS_getpgid __NR_getpgid
#define SYS_getpgrp __NR_getpgrp
#define SYS_getpid __NR_getpid
#define SYS_getpmsg __NR_getpmsg
#define SYS_getppid __NR_getppid
#define SYS_getpriority __NR_getpriority
#define SYS_getrandom __NR_getrandom
#define SYS_getresgid __NR_getresgid
#define SYS_getresgid32 __NR_getresgid32
#define SYS_getresuid __NR_getresuid
#define SYS_getresuid32 __NR_getresuid32
#define SYS_getrlimit __NR_getrlimit
#define SYS_get_robust_list __NR_get_robust_list
#define SYS_getrusage __NR_getrusage
#define SYS_getsid __NR_getsid
#define SYS_getsockname __NR_getsockname
#define SYS_getsockopt __NR_getsockopt
#define SYS_get_thread_area __NR_get_thread_area
#define SYS_gettid __NR_gettid
#define SYS_gettimeofday __NR_gettimeofday
#define SYS_getuid __NR_getuid
#define SYS_getuid32 __NR_getuid32
#define SYS_getunwind __NR_getunwind
#define SYS_getxattr __NR_getxattr
#define SYS_getxgid __NR_getxgid
#define SYS_getxpid __NR_getxpid
#define SYS_getxuid __NR_getxuid
#define SYS_gtty __NR_gtty
#define SYS_ia32_arch_prctl __NR_ia32_arch_prctl
#define SYS_ia32_io_pgetevents __NR_ia32_io_pgetevents
#define SYS_ia32_rseq __NR_ia32_rseq
#define SYS_ia32_statx __NR_ia32_statx
#define SYS_idle __NR_idle
#define SYS_init_module __NR_init_module
#define SYS_inotify_add_watch __NR_inotify_add_watch
#define SYS_inotify_init __NR_inotify_init
#define SYS_inotify_init1 __NR_inotify_init1
#define SYS_inotify_rm_watch __NR_inotify_rm_watch
#define SYS_io_cancel __NR_io_cancel
#define SYS_ioctl __NR_ioctl
#define SYS_io_destroy __NR_io_destroy
#define SYS_io_getevents __NR_io_getevents
#define SYS_ioperm __NR_ioperm
#define SYS_io_pgetevents __NR_io_pgetevents
#define SYS_io_pgetevents_time64 __NR_io_pgetevents_time64
#define SYS_iopl __NR_iopl
#define SYS_ioprio_get __NR_ioprio_get
#define SYS_ioprio_set __NR_ioprio_set
#define SYS_io_setup __NR_io_setup
#define SYS_io_submit __NR_io_submit
#define SYS_io_uring_enter __NR_io_uring_enter
#define SYS_io_uring_register __NR_io_uring_register
#define SYS_io_uring_setup __NR_io_uring_setup
#define SYS_ipc __NR_ipc
#define SYS_kcmp __NR_kcmp
#define SYS_kern_features __NR_kern_features
#define SYS_kexec_file_load __NR_kexec_file_load
#define SYS_kexec_load __NR_kexec_load
#define SYS_keyctl __NR_keyctl
#define SYS_kill __NR_kill
#define SYS_landlock_add_rule __NR_landlock_add_rule
#define SYS_landlock_create_ruleset __NR_landlock_create_ruleset
#define SYS_landlock_restrict_self __NR_landlock_restrict_self
#define SYS_lchown __NR_lchown
#define SYS_lchown32 __NR_lchown32
#define SYS_lgetxattr __NR_lgetxattr
#define SYS_link __NR_link
#define SYS_linkat __NR_linkat
#define SYS_Linux __NR_Linux
#define SYS_Linux_syscalls __NR_Linux_syscalls
#define SYS_listen __NR_listen
#define SYS_listxattr __NR_listxattr
#define SYS_llistxattr __NR_llistxattr
#define SYS__llseek __NR__llseek
#define SYS_lock __NR_lock
#define SYS_lookup_dcookie __NR_lookup_dcookie
#define SYS_lremovexattr __NR_lremovexattr
#define SYS_lseek __NR_lseek
#define SYS_lsetxattr __NR_lsetxattr
#define SYS_lstat __NR_lstat
#define SYS_lstat64 __NR_lstat64
#define SYS_madvise __NR_madvise
#define SYS_madvise1 __NR_madvise1
#define SYS_mbind __NR_mbind
#define SYS_membarrier __NR_membarrier
#define SYS_memfd_create __NR_memfd_create
#define SYS_memory_ordering __NR_memory_ordering
#define SYS_migrate_pages __NR_migrate_pages
#define SYS_mincore __NR_mincore
#define SYS_mkdir __NR_mkdir
#define SYS_mkdirat __NR_mkdirat
#define SYS_mknod __NR_mknod
#define SYS_mknodat __NR_mknodat
#define SYS_mlock __NR_mlock
#define SYS_mlock2 __NR_mlock2
#define SYS_mlockall __NR_mlockall
#define SYS_mmap __NR_mmap
#define SYS_mmap2 __NR_mmap2
#define SYS_modify_ldt __NR_modify_ldt
#define SYS_mount __NR_mount
#define SYS_mount_setattr __NR_mount_setattr
#define SYS_move_mount __NR_move_mount
#define SYS_move_pages __NR_move_pages
#define SYS_mprotect __NR_mprotect
#define SYS_mpx __NR_mpx
#define SYS_mq_getsetattr __NR_mq_getsetattr
#define SYS_mq_notify __NR_mq_notify
#define SYS_mq_open __NR_mq_open
#define SYS_mq_timedreceive __NR_mq_timedreceive
#define SYS_mq_timedreceive_time64 __NR_mq_timedreceive_time64
#define SYS_mq_timedsend __NR_mq_timedsend
#define SYS_mq_timedsend_time64 __NR_mq_timedsend_time64
#define SYS_mq_unlink __NR_mq_unlink
#define SYS_mremap __NR_mremap
#define SYS_msgctl __NR_msgctl
#define SYS_msgget __NR_msgget
#define SYS_msgrcv __NR_msgrcv
#define SYS_msgsnd __NR_msgsnd
#define SYS_msync __NR_msync
#define SYS_multiplexer __NR_multiplexer
#define SYS_munlock __NR_munlock
#define SYS_munlockall __NR_munlockall
#define SYS_munmap __NR_munmap
#define SYS_name_to_handle_at __NR_name_to_handle_at
#define SYS_nanosleep __NR_nanosleep
#define SYS_newfstatat __NR_newfstatat
#define SYS__newselect __NR__newselect
#define SYS_nfsservctl __NR_nfsservctl
#define SYS_nice __NR_nice
#define SYS_ni_syscall __NR_ni_syscall
#define SYS_OABI_SYSCALL_BASE __NR_OABI_SYSCALL_BASE
#define SYS_old_adjtimex __NR_old_adjtimex
#define SYS_oldfstat __NR_oldfstat
#define SYS_old_getpagesize __NR_old_getpagesize
#define SYS_oldlstat __NR_oldlstat
#define SYS_oldolduname __NR_oldolduname
#define SYS_oldstat __NR_oldstat
#define SYS_oldumount __NR_oldumount
#define SYS_olduname __NR_olduname
#define SYS_open __NR_open
#define SYS_openat __NR_openat
#define SYS_openat2 __NR_openat2
#define SYS_open_by_handle_at __NR_open_by_handle_at
#define SYS_open_tree __NR_open_tree
#define SYS_osf_adjtime __NR_osf_adjtime
#define SYS_osf_afs_syscall __NR_osf_afs_syscall
#define SYS_osf_alt_plock __NR_osf_alt_plock
#define SYS_osf_alt_setsid __NR_osf_alt_setsid
#define SYS_osf_alt_sigpending __NR_osf_alt_sigpending
#define SYS_osf_asynch_daemon __NR_osf_asynch_daemon
#define SYS_osf_audcntl __NR_osf_audcntl
#define SYS_osf_audgen __NR_osf_audgen
#define SYS_osf_chflags __NR_osf_chflags
#define SYS_osf_execve __NR_osf_execve
#define SYS_osf_exportfs __NR_osf_exportfs
#define SYS_osf_fchflags __NR_osf_fchflags
#define SYS_osf_fdatasync __NR_osf_fdatasync
#define SYS_osf_fpathconf __NR_osf_fpathconf
#define SYS_osf_fstatfs __NR_osf_fstatfs
#define SYS_osf_fuser __NR_osf_fuser
#define SYS_osf_getaddressconf __NR_osf_getaddressconf
#define SYS_osf_getdirentries __NR_osf_getdirentries
#define SYS_osf_getdomainname __NR_osf_getdomainname
#define SYS_osf_getfh __NR_osf_getfh
#define SYS_osf_getfsstat __NR_osf_getfsstat
#define SYS_osf_gethostid __NR_osf_gethostid
#define SYS_osf_getitimer __NR_osf_getitimer
#define SYS_osf_getlogin __NR_osf_getlogin
#define SYS_osf_getmnt __NR_osf_getmnt
#define SYS_osf_getrusage __NR_osf_getrusage
#define SYS_osf_getsysinfo __NR_osf_getsysinfo
#define SYS_osf_gettimeofday __NR_osf_gettimeofday
#define SYS_osf_kloadcall __NR_osf_kloadcall
#define SYS_osf_kmodcall __NR_osf_kmodcall
#define SYS_osf_memcntl __NR_osf_memcntl
#define SYS_osf_mincore __NR_osf_mincore
#define SYS_osf_mount __NR_osf_mount
#define SYS_osf_mremap __NR_osf_mremap
#define SYS_osf_msfs_syscall __NR_osf_msfs_syscall
#define SYS_osf_msleep __NR_osf_msleep
#define SYS_osf_mvalid __NR_osf_mvalid
#define SYS_osf_mwakeup __NR_osf_mwakeup
#define SYS_osf_naccept __NR_osf_naccept
#define SYS_osf_nfssvc __NR_osf_nfssvc
#define SYS_osf_ngetpeername __NR_osf_ngetpeername
#define SYS_osf_ngetsockname __NR_osf_ngetsockname
#define SYS_osf_nrecvfrom __NR_osf_nrecvfrom
#define SYS_osf_nrecvmsg __NR_osf_nrecvmsg
#define SYS_osf_nsendmsg __NR_osf_nsendmsg
#define SYS_osf_ntp_adjtime __NR_osf_ntp_adjtime
#define SYS_osf_ntp_gettime __NR_osf_ntp_gettime
#define SYS_osf_old_creat __NR_osf_old_creat
#define SYS_osf_old_fstat __NR_osf_old_fstat
#define SYS_osf_old_getpgrp __NR_osf_old_getpgrp
#define SYS_osf_old_killpg __NR_osf_old_killpg
#define SYS_osf_old_lstat __NR_osf_old_lstat
#define SYS_osf_old_open __NR_osf_old_open
#define SYS_osf_oldquota __NR_osf_oldquota
#define SYS_osf_old_sigaction __NR_osf_old_sigaction
#define SYS_osf_old_sigblock __NR_osf_old_sigblock
#define SYS_osf_old_sigreturn __NR_osf_old_sigreturn
#define SYS_osf_old_sigsetmask __NR_osf_old_sigsetmask
#define SYS_osf_old_sigvec __NR_osf_old_sigvec
#define SYS_osf_old_stat __NR_osf_old_stat
#define SYS_osf_old_vadvise __NR_osf_old_vadvise
#define SYS_osf_old_vtrace __NR_osf_old_vtrace
#define SYS_osf_old_wait __NR_osf_old_wait
#define SYS_osf_pathconf __NR_osf_pathconf
#define SYS_osf_pid_block __NR_osf_pid_block
#define SYS_osf_pid_unblock __NR_osf_pid_unblock
#define SYS_osf_plock __NR_osf_plock
#define SYS_osf_priocntlset __NR_osf_priocntlset
#define SYS_osf_profil __NR_osf_profil
#define SYS_osf_proplist_syscall __NR_osf_proplist_syscall
#define SYS_osf_reboot __NR_osf_reboot
#define SYS_osf_revoke __NR_osf_revoke
#define SYS_osf_sbrk __NR_osf_sbrk
#define SYS_osf_security __NR_osf_security
#define SYS_osf_select __NR_osf_select
#define SYS_osf_sethostid __NR_osf_sethostid
#define SYS_osf_setitimer __NR_osf_setitimer
#define SYS_osf_setlogin __NR_osf_setlogin
#define SYS_osf_set_program_attributes __NR_osf_set_program_attributes
#define SYS_osf_set_speculative __NR_osf_set_speculative
#define SYS_osf_setsysinfo __NR_osf_setsysinfo
#define SYS_osf_settimeofday __NR_osf_settimeofday
#define SYS_osf_shmat __NR_osf_shmat
#define SYS_osf_signal __NR_osf_signal
#define SYS_osf_sigprocmask __NR_osf_sigprocmask
#define SYS_osf_sigsendset __NR_osf_sigsendset
#define SYS_osf_sigstack __NR_osf_sigstack
#define SYS_osf_sigwaitprim __NR_osf_sigwaitprim
#define SYS_osf_sstk __NR_osf_sstk
#define SYS_osf_statfs __NR_osf_statfs
#define SYS_osf_subsys_info __NR_osf_subsys_info
#define SYS_osf_swapctl __NR_osf_swapctl
#define SYS_osf_swapon __NR_osf_swapon
#define SYS_osf_syscall __NR_osf_syscall
#define SYS_osf_sysinfo __NR_osf_sysinfo
#define SYS_osf_table __NR_osf_table
#define SYS_osf_uadmin __NR_osf_uadmin
#define SYS_osf_usleep_thread __NR_osf_usleep_thread
#define SYS_osf_uswitch __NR_osf_uswitch
#define SYS_osf_utc_adjtime __NR_osf_utc_adjtime
#define SYS_osf_utc_gettime __NR_osf_utc_gettime
#define SYS_osf_utimes __NR_osf_utimes
#define SYS_osf_utsname __NR_osf_utsname
#define SYS_osf_wait4 __NR_osf_wait4
#define SYS_osf_waitid __NR_osf_waitid
#define SYS_pause __NR_pause
#define SYS_pciconfig_iobase __NR_pciconfig_iobase
#define SYS_pciconfig_read __NR_pciconfig_read
#define SYS_pciconfig_write __NR_pciconfig_write
#define SYS_perfctr __NR_perfctr
#define SYS_perf_event_open __NR_perf_event_open
#define SYS_perfmonctl __NR_perfmonctl
#define SYS_personality __NR_personality
#define SYS_pidfd_getfd __NR_pidfd_getfd
#define SYS_pidfd_open __NR_pidfd_open
#define SYS_pidfd_send_signal __NR_pidfd_send_signal
#define SYS_pipe __NR_pipe
#define SYS_pipe2 __NR_pipe2
#define SYS_pivot_root __NR_pivot_root
#define SYS_pkey_alloc __NR_pkey_alloc
#define SYS_pkey_free __NR_pkey_free
#define SYS_pkey_mprotect __NR_pkey_mprotect
#define SYS_poll __NR_poll
#define SYS_ppoll __NR_ppoll
#define SYS_ppoll_time64 __NR_ppoll_time64
#define SYS_prctl __NR_prctl
#define SYS_pread __NR_pread
#define SYS_pread64 __NR_pread64
#define SYS_preadv __NR_preadv
#define SYS_preadv2 __NR_preadv2
#define SYS_prlimit64 __NR_prlimit64
#define SYS_process_madvise __NR_process_madvise
#define SYS_process_vm_readv __NR_process_vm_readv
#define SYS_process_vm_writev __NR_process_vm_writev
#define SYS_prof __NR_prof
#define SYS_profil __NR_profil
#define SYS_pselect6 __NR_pselect6
#define SYS_pselect6_time64 __NR_pselect6_time64
#define SYS_ptrace __NR_ptrace
#define SYS_putmsg __NR_putmsg
#define SYS_putpmsg __NR_putpmsg
#define SYS_pwrite __NR_pwrite
#define SYS_pwrite64 __NR_pwrite64
#define SYS_pwritev __NR_pwritev
#define SYS_pwritev2 __NR_pwritev2
#define SYS_query_module __NR_query_module
#define SYS_quotactl __NR_quotactl
#define SYS_read __NR_read
#define SYS_readahead __NR_readahead
#define SYS_readdir __NR_readdir
#define SYS_readlink __NR_readlink
#define SYS_readlinkat __NR_readlinkat
#define SYS_readv __NR_readv
#define SYS_reboot __NR_reboot
#define SYS_recv __NR_recv
#define SYS_recvfrom __NR_recvfrom
#define SYS_recvmmsg __NR_recvmmsg
#define SYS_recvmmsg_time64 __NR_recvmmsg_time64
#define SYS_recvmsg __NR_recvmsg
#define SYS_remap_file_pages __NR_remap_file_pages
#define SYS_removexattr __NR_removexattr
#define SYS_rename __NR_rename
#define SYS_renameat __NR_renameat
#define SYS_renameat2 __NR_renameat2
#define SYS_request_key __NR_request_key
#define SYS_reserved177 __NR_reserved177
#define SYS_reserved193 __NR_reserved193
#define SYS_reserved221 __NR_reserved221
#define SYS_reserved82 __NR_reserved82
#define SYS_restart_syscall __NR_restart_syscall
#define SYS_riscv_flush_icache __NR_riscv_flush_icache
#define SYS_rmdir __NR_rmdir
#define SYS_rseq __NR_rseq
#define SYS_rtas __NR_rtas
#define SYS_rt_sigaction __NR_rt_sigaction
#define SYS_rt_sigpending __NR_rt_sigpending
#define SYS_rt_sigprocmask __NR_rt_sigprocmask
#define SYS_rt_sigqueueinfo __NR_rt_sigqueueinfo
#define SYS_rt_sigreturn __NR_rt_sigreturn
#define SYS_rt_sigsuspend __NR_rt_sigsuspend
#define SYS_rt_sigtimedwait __NR_rt_sigtimedwait
#define SYS_rt_sigtimedwait_time64 __NR_rt_sigtimedwait_time64
#define SYS_rt_tgsigqueueinfo __NR_rt_tgsigqueueinfo
#define SYS_s390_guarded_storage __NR_s390_guarded_storage
#define SYS_s390_pci_mmio_read __NR_s390_pci_mmio_read
#define SYS_s390_pci_mmio_write __NR_s390_pci_mmio_write
#define SYS_s390_runtime_instr __NR_s390_runtime_instr
#define SYS_s390_sthyi __NR_s390_sthyi
#define SYS_sched_getaffinity __NR_sched_getaffinity
#define SYS_sched_getattr __NR_sched_getattr
#define SYS_sched_getparam __NR_sched_getparam
#define SYS_sched_get_priority_max __NR_sched_get_priority_max
#define SYS_sched_get_priority_min __NR_sched_get_priority_min
#define SYS_sched_getscheduler __NR_sched_getscheduler
#define SYS_sched_rr_get_interval __NR_sched_rr_get_interval
#define SYS_sched_rr_get_interval_time64 __NR_sched_rr_get_interval_time64
#define SYS_sched_setaffinity __NR_sched_setaffinity
#define SYS_sched_setattr __NR_sched_setattr
#define SYS_sched_setparam __NR_sched_setparam
#define SYS_sched_setscheduler __NR_sched_setscheduler
#define SYS_sched_yield __NR_sched_yield
#define SYS_seccomp __NR_seccomp
#define SYS_security __NR_security
#define SYS_select __NR_select
#define SYS_semctl __NR_semctl
#define SYS_semget __NR_semget
#define SYS_semop __NR_semop
#define SYS_semtimedop __NR_semtimedop
#define SYS_semtimedop_time64 __NR_semtimedop_time64
#define SYS_send __NR_send
#define SYS_sendfile __NR_sendfile
#define SYS_sendfile64 __NR_sendfile64
#define SYS_sendmmsg __NR_sendmmsg
#define SYS_sendmsg __NR_sendmsg
#define SYS_sendto __NR_sendto
#define SYS_setdomainname __NR_setdomainname
#define SYS_setfsgid __NR_setfsgid
#define SYS_setfsgid32 __NR_setfsgid32
#define SYS_setfsuid __NR_setfsuid
#define SYS_setfsuid32 __NR_setfsuid32
#define SYS_setgid __NR_setgid
#define SYS_setgid32 __NR_setgid32
#define SYS_setgroups __NR_setgroups
#define SYS_setgroups32 __NR_setgroups32
#define SYS_sethae __NR_sethae
#define SYS_sethostname __NR_sethostname
#define SYS_setitimer __NR_setitimer
#define SYS_set_mempolicy __NR_set_mempolicy
#define SYS_setns __NR_setns
#define SYS_setpgid __NR_setpgid
#define SYS_setpgrp __NR_setpgrp
#define SYS_setpriority __NR_setpriority
#define SYS_setregid __NR_setregid
#define SYS_setregid32 __NR_setregid32
#define SYS_setresgid __NR_setresgid
#define SYS_setresgid32 __NR_setresgid32
#define SYS_setresuid __NR_setresuid
#define SYS_setresuid32 __NR_setresuid32
#define SYS_setreuid __NR_setreuid
#define SYS_setreuid32 __NR_setreuid32
#define SYS_setrlimit __NR_setrlimit
#define SYS_set_robust_list __NR_set_robust_list
#define SYS_setsid __NR_setsid
#define SYS_setsockopt __NR_setsockopt
#define SYS_set_thread_area __NR_set_thread_area
#define SYS_set_tid_address __NR_set_tid_address
#define SYS_settimeofday __NR_settimeofday
#define SYS_setuid __NR_setuid
#define SYS_setuid32 __NR_setuid32
#define SYS_setxattr __NR_setxattr
#define SYS_set_zone_reclaim __NR_set_zone_reclaim
#define SYS_sgetmask __NR_sgetmask
#define SYS_shmat __NR_shmat
#define SYS_shmctl __NR_shmctl
#define SYS_shmdt __NR_shmdt
#define SYS_shmget __NR_shmget
#define SYS_shutdown __NR_shutdown
#define SYS_sigaction __NR_sigaction
#define SYS_sigaltstack __NR_sigaltstack
#define SYS_signal __NR_signal
#define SYS_signalfd __NR_signalfd
#define SYS_signalfd4 __NR_signalfd4
#define SYS_sigpending __NR_sigpending
#define SYS_sigprocmask __NR_sigprocmask
#define SYS_sigreturn __NR_sigreturn
#define SYS_sigsuspend __NR_sigsuspend
#define SYS_socket __NR_socket
#define SYS_socketcall __NR_socketcall
#define SYS_socketpair __NR_socketpair
#define SYS_splice __NR_splice
#define SYS_spu_create __NR_spu_create
#define SYS_spu_run __NR_spu_run
#define SYS_ssetmask __NR_ssetmask
#define SYS_stat __NR_stat
#define SYS_stat64 __NR_stat64
#define SYS_statfs __NR_statfs
#define SYS_statfs64 __NR_statfs64
#define SYS_statx __NR_statx
#define SYS_stime __NR_stime
#define SYS_stty __NR_stty
#define SYS_subpage_prot __NR_subpage_prot
#define SYS_swapcontext __NR_swapcontext
#define SYS_swapoff __NR_swapoff
#define SYS_swapon __NR_swapon
#define SYS_switch_endian __NR_switch_endian
#define SYS_symlink __NR_symlink
#define SYS_symlinkat __NR_symlinkat
#define SYS_sync __NR_sync
#define SYS_sync_file_range __NR_sync_file_range
#define SYS_sync_file_range2 __NR_sync_file_range2
#define SYS_syncfs __NR_syncfs
#define SYS_syscall __NR_syscall
#define SYS_SYSCALL_BASE __NR_SYSCALL_BASE
#define SYS__sysctl __NR__sysctl
#define SYS_sys_debug_setcontext __NR_sys_debug_setcontext
#define SYS_sys_epoll_create __NR_sys_epoll_create
#define SYS_sys_epoll_ctl __NR_sys_epoll_ctl
#define SYS_sys_epoll_wait __NR_sys_epoll_wait
#define SYS_sysfs __NR_sysfs
#define SYS_sysinfo __NR_sysinfo
#define SYS_sys_kexec_load __NR_sys_kexec_load
#define SYS_syslog __NR_syslog
#define SYS_sysmips __NR_sysmips
#define SYS_sysriscv __NR_sysriscv
#define SYS_sys_setaltroot __NR_sys_setaltroot
#define SYS_tee __NR_tee
#define SYS_tgkill __NR_tgkill
#define SYS_time __NR_time
#define SYS_timer_create __NR_timer_create
#define SYS_timer_delete __NR_timer_delete
#define SYS_timerfd __NR_timerfd
#define SYS_timerfd_create __NR_timerfd_create
#define SYS_timerfd_gettime __NR_timerfd_gettime
#define SYS_timerfd_gettime64 __NR_timerfd_gettime64
#define SYS_timerfd_settime __NR_timerfd_settime
#define SYS_timerfd_settime64 __NR_timerfd_settime64
#define SYS_timer_getoverrun __NR_timer_getoverrun
#define SYS_timer_gettime __NR_timer_gettime
#define SYS_timer_gettime64 __NR_timer_gettime64
#define SYS_timer_settime __NR_timer_settime
#define SYS_timer_settime64 __NR_timer_settime64
#define SYS_times __NR_times
#define SYS_tkill __NR_tkill
#define SYS_truncate __NR_truncate
#define SYS_truncate64 __NR_truncate64
#define SYS_tuxcall __NR_tuxcall
#define SYS_ugetrlimit __NR_ugetrlimit
#define SYS_ulimit __NR_ulimit
#define SYS_umask __NR_umask
#define SYS_umount __NR_umount
#define SYS_umount2 __NR_umount2
#define SYS_umount_with_flags __NR_umount_with_flags
#define SYS_umount_without_flags __NR_umount_without_flags
#define SYS_uname __NR_uname
#define SYS_unlink __NR_unlink
#define SYS_unlinkat __NR_unlinkat
#define SYS_unshare __NR_unshare
#define SYS_unused109 __NR_unused109
#define SYS_unused150 __NR_unused150
#define SYS_unused18 __NR_unused18
#define SYS_unused28 __NR_unused28
#define SYS_unused59 __NR_unused59
#define SYS_unused84 __NR_unused84
#define SYS_uselib __NR_uselib
#define SYS_userfaultfd __NR_userfaultfd
#define SYS_ustat __NR_ustat
#define SYS_utime __NR_utime
#define SYS_utimensat __NR_utimensat
#define SYS_utimensat_time64 __NR_utimensat_time64
#define SYS_utimes __NR_utimes
#define SYS_utrap_install __NR_utrap_install
#define SYS_vfork __NR_vfork
#define SYS_vhangup __NR_vhangup
#define SYS_vm86 __NR_vm86
#define SYS_vm86old __NR_vm86old
#define SYS_vmsplice __NR_vmsplice
#define SYS_vserver __NR_vserver
#define SYS_wait4 __NR_wait4
#define SYS_waitid __NR_waitid
#define SYS_waitpid __NR_waitpid
#define SYS_write __NR_write
#define SYS_writev __NR_writev
